MerkleProofSpec

Trait MerkleProofSpec 

pub trait MerkleProofSpec {
    type Witness: Witness + Send + Sync;
    type Hasher: Digest<OutputSize = U32>;
}
A trait specifying the hash function and format of the witness used in merkle proofs for storage access

Required Associated Types

type Witness: Witness + Send + Sync

The structure that accumulates the witness data

type Hasher: Digest<OutputSize = U32>

The hash function used to compute the merkle root

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors