pub trait MerkleProofSpec {
type Witness: Witness + Send + Sync;
type Hasher: Digest<OutputSize = U32>;
}Expand description
A trait specifying the hash function and format of the witness used in merkle proofs for storage access
Required Associated Types
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".